#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include <iostream>
#include <string>
#include "LHAPDF/LHAPDF.h"
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;
using namespace LHAPDF;

/*
	We calculate the momentum sum rule of a parton density function by performing explicitly the integral.
      We use the LHAPDF library to access the different PDF parameterisations.
      We chach the momentum sum rule for a standard set, adn for the LO* parameterisations.

      We use LHAPDF 5.7.0

	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      
*/

int main()
{
  	const int SUBSET = 0;
  	const string NAME = "MRST2004nlo";
//  	const string NAME = "MRST2007lomod";		// LO * pdf //
//  	const string NAME = "MRSTMCal";			// LO** pdf //
  	LHAPDF::initPDFSet(NAME, LHAPDF::LHGRID, SUBSET);
#define LVEC 2
	int n1, npoints = 1000000;
	double rvec[LVEC];
  	double x = 0.01, Q = 10.0, Q2 = 100.0, mz=91.2;
      double sum0=0, sum00=0, f, ff, sigma2, error;
// initialise random number generator
//    rlxd_init( luxory level, seed )
      rlxd_init(2,32767);

  	cout << "alphas(mz) = " << LHAPDF::alphasPDF(mz) << endl;
  	cout << "qcdlam4    = " << LHAPDF::getLam4(SUBSET) << endl;
  	cout << "qcdlam5    = " << LHAPDF::getLam5(SUBSET) << endl;
  	cout << "orderPDF   = " << LHAPDF::getOrderPDF() << endl;
  	cout << "xmin       = " << LHAPDF::getXmin(SUBSET) << endl;
  	cout << "xmax       = " << LHAPDF::getXmax(SUBSET) << endl;
  	cout << "q2min      = " << LHAPDF::getQ2min(SUBSET) << endl;
  	cout << "q2max      = " << LHAPDF::getQ2max(SUBSET) << endl;
  	cout << "orderalfas = " << LHAPDF::getOrderAlphaS() << endl;
  	cout << "num flav   = " << LHAPDF::getNf() << endl;
  	cout << "name       = " << NAME << endl;
  	cout << "number     = " << LHAPDF::numberPDF() << endl;
  	cout << endl;

//  	const int NUMBER = numberPDF();



    	initPDF(0);

      double xmin = getXmin(0);
      double xmax = getXmax(0);

      Q = 10.0;
      Q2 = Q*Q;
	for ( n1 = 0; n1 < npoints; n1++ ) {
      	ranlxd(rvec,LVEC);
// for simple integration
// here 	get x from linear sampling
// for importance sampling
// here get x from importance sampling
            f=0;
// here calculate the sum rule ...
// the pdf from LHAPDFLIB is called via: xfx(x,Q,flavor) 
//		with x=fractional momentum
//		     Q = sqrt(q2), the sqrt of the scale
//		     flavor = -6,.. ,6 the flavor code of the parton, 
//		     flavor: 0=gluon, 1=doqn, 2=up, 3=strange, 4=charm, 5=bottom 6=top           
		sum0 = sum0 + ff;
            sum00 = sum00 + ff*ff; 
//                                  
	}
      
      sum0 = sum0/npoints;
      sum00 = sum00/npoints;
      sigma2 = sum00 - sum0*sum0 ;
      error = sqrt(sigma2/npoints) ;
      cout<<" sum rule is: "<<sum0<<"+/-"<< error<<endl;
 
	return(0);

}
